@@ -3,7 +3,6 @@
 #include "observer.hpp"
 #include "taskflow.hpp"
 #include "async_task.hpp"
-#include "freelist.hpp"
 
 /**
 @file executor.hpp
@@ -67,6 +66,7 @@ class Executor {
   friend class Runtime;
   friend class NonpreemptiveRuntime;
   friend class Algorithm;
+  friend class TaskGroup;
 
   public:
 
@@ -74,7 +74,7 @@ class Executor {
   @brief constructs the executor with @c N worker threads
 
   @param N number of workers (default std::thread::hardware_concurrency)
-  @param wix interface class instance to configure workers' behaviors
+  @param wif interface class instance to configure workers' behaviors
 
   The constructor spawns @c N worker threads to run tasks in a
   work-stealing loop. The number of workers must be greater than zero
@@ -90,7 +90,7 @@ class Executor {
   */
   explicit Executor(
     size_t N = std::thread::hardware_concurrency(),
-    std::unique_ptr<WorkerInterface> wix = nullptr
+    std::unique_ptr<WorkerInterface> wif = nullptr
   );
 
   /**
@@ -468,7 +468,6 @@ class Executor {
   participating in the work-stealing loop, executing available tasks alongside 
   other workers.  
 
-
   @code{.cpp}
   taskflow.emplace([&](){
     std::future<void> fu = std::async([](){ std::sleep(100s); });
@@ -503,8 +502,7 @@ class Executor {
   /**
   @brief queries the number of worker threads
 
-  Each worker represents one unique thread spawned by an executor
-  upon its construction time.
+  Each worker represents a unique thread spawned by an executor upon its construction time.
 
   @code{.cpp}
   tf::Executor executor(4);
@@ -514,12 +512,15 @@ class Executor {
   size_t num_workers() const noexcept;
   
   /**
-  @brief queries the number of workers that are currently not making any stealing attempts
+  @brief queries the number of workers that are in the waiting loop
+
+  A worker in the waiting loop has exhausted its local queue and made enough stealing attempts,
+  and is now ready to be preempted and enter the waiting state.
   */
   size_t num_waiters() const noexcept;
   
   /**
-  @brief queries the number of queues used in the work-stealing loop
+  @brief queries the number of work-stealing queues used by the executor
   */
   size_t num_queues() const noexcept;
 
@@ -1065,26 +1066,76 @@ class Executor {
   >
   auto dependent_async(P&& params, F&& func, I first, I last);
 
+  // ----------------------------------------------------------------------------------------------
+  // Task Group
+  // ----------------------------------------------------------------------------------------------
+  
+  /**
+  @brief creates a task group that executes a collection of asynchronous tasks
+  @return a tf::TaskGroup object associated with the current executor
+ 
+  A TaskGroup allows submitting multiple asynchronous tasks to the executor
+  and waiting for their completion collectively using `corun()`. Tasks added
+  to the group can execute in parallel and may capture local variables by value
+  or reference, depending on your needs. 
+  This can be useful for divide-and-conquer
+  algorithms, parallel loops, or any workflow that requires grouping related tasks.
+  
+  Example (computing Fibonacci numbers in parallel):
+
+  @code{.cpp}
+  tf::Executor executor;
+  
+  size_t fibonacci(size_t N) {
+
+    if (N < 2) return N;
+  
+    size_t res1, res2;
+  
+    // Create a task group from the current executor
+    tf::TaskGroup tg = get_executor().task_group();
+  
+    // Submit asynchronous tasks to the group
+    tg.silent_async([N, &res1](){ res1 = fibonacci(N-1); });
+    res2 = fibonacci(N-2);  // compute one branch synchronously
+  
+    // Wait for all tasks in the group to complete
+    tg.corun();
+  
+    return res1 + res2;
+  }
+  
+  int main() {
+    return executor.async([](){ return fibonacci(30); }).get();
+  }
+  @endcode
+  
+  This member function is thread-safe.
+
+  @attention
+  Due to cooperative execution, a task group can only be created by a worker of an executor.
+  */
+  TaskGroup task_group();
+
   private:
-    
+  
+  struct Buffer {
+    std::mutex mutex;
+    UnboundedWSQ<Node*> queue;
+  };  
+
   std::mutex _taskflows_mutex;
   
   std::vector<Worker> _workers;
   DefaultNotifier _notifier;
 
-#if __cplusplus >= TF_CPP20
   std::atomic<size_t> _num_topologies {0};
-#else
-  std::condition_variable _topology_cv;
-  std::mutex _topology_mutex;
-  size_t _num_topologies {0};
-#endif
   
   std::list<Taskflow> _taskflows;
 
-  Freelist<Node*> _buffers;
+  std::vector<Buffer> _buffers;
 
-  std::unique_ptr<WorkerInterface> _worker_interface;
+  std::unique_ptr<WorkerInterface> _worker_if;
   std::unordered_set<std::shared_ptr<ObserverInterface>> _observers;
   std::unordered_map<std::thread::id, Worker*> _t2w;
 
@@ -1096,6 +1147,7 @@ class Executor {
   bool _explore_task(Worker&, Node*&);
   void _schedule(Worker&, Node*);
   void _schedule(Node*);
+  void _spill(Node*);
   void _set_up_topology(Worker*, Topology*);
   void _tear_down_topology(Worker&, Topology*);
   void _tear_down_async(Worker&, Node*, Node*&);
@@ -1125,48 +1177,51 @@ class Executor {
   bool _invoke_runtime_task_impl(Worker&, Node*, std::function<void(Runtime&, bool)>&);
 
   template <typename I>
-  I _set_up_graph(I, I, Topology*, Node*);
+  I _set_up_graph(I, I, Topology*, NodeBase*);
   
   template <typename P>
   void _corun_until(Worker&, P&&);
   
   template <typename I>
-  void _corun_graph(Worker&, Node*, I, I);
+  void _corun_graph(Worker&, Topology*, NodeBase*, I, I);
 
   template <typename I>
-  void _schedule(Worker&, I, I);
+  void _bulk_schedule(Worker&, I, size_t);
 
   template <typename I>
-  void _schedule(I, I);
+  void _bulk_schedule(I, size_t);
 
   template <typename I>
-  void _schedule_graph_with_parent(Worker&, I, I, Node*);
+  void _bulk_spill(I, size_t);
+
+  template <typename I>
+  void _schedule_graph_with_parent(Worker&, I, I, Topology*, NodeBase*);
 
   template <typename P, typename F>
-  auto _async(P&&, F&&, Topology*, Node*);
+  auto _async(P&&, F&&, Topology*, NodeBase*);
 
   template <typename P, typename F>
-  void _silent_async(P&&, F&&, Topology*, Node*);
+  void _silent_async(P&&, F&&, Topology*, NodeBase*);
 
   template <typename P, typename F, typename I,
     std::enable_if_t<is_task_params_v<P> && !std::is_same_v<std::decay_t<I>, AsyncTask>, void>* = nullptr
   >
-  auto _dependent_async(P&&, F&&, I, I, Topology*, Node*);
+  auto _dependent_async(P&&, F&&, I, I, Topology*, NodeBase*);
   
   template <typename P, typename F, typename I, 
     std::enable_if_t<is_task_params_v<P> && !std::is_same_v<std::decay_t<I>, AsyncTask>, void>* = nullptr
   >
-  auto _silent_dependent_async(P&&, F&&, I, I, Topology*, Node*);
+  auto _silent_dependent_async(P&&, F&&, I, I, Topology*, NodeBase*);
 };
 
 #ifndef DOXYGEN_GENERATING_OUTPUT
 
 // Constructor
-inline Executor::Executor(size_t N, std::unique_ptr<WorkerInterface> wix) :
+inline Executor::Executor(size_t N, std::unique_ptr<WorkerInterface> wif) :
   _workers  (N),
-  _notifier (N),
-  _buffers  (N),
-  _worker_interface(std::move(wix)) {
+  _notifier (N),                
+  _buffers  (std::bit_width(N)), // Empirically, we find that log2(N) performs best.
+  _worker_if(std::move(wif)) {
 
   if(N == 0) {
     TF_THROW("executor must define at least one worker");
@@ -1205,11 +1260,7 @@ inline void Executor::_shutdown() {
 
   // shut down the scheduler
   for(size_t i=0; i<_workers.size(); ++i) {
-  #if __cplusplus >= TF_CPP20
     _workers[i]._done.test_and_set(std::memory_order_relaxed);
-  #else
-    _workers[i]._done.store(true, std::memory_order_relaxed);
-  #endif
   }
   
   _notifier.notify_all();
@@ -1230,13 +1281,7 @@ inline size_t Executor::num_workers() const noexcept {
 
 // Function: num_waiters
 inline size_t Executor::num_waiters() const noexcept {
-#if __cplusplus >= TF_CPP20
   return _notifier.num_waiters();
-#else
-  // Unfortunately, nonblocking notifier does not have an easy way to return
-  // the number of workers that are not making stealing attempts.
-  return 0;
-#endif
 }
 
 // Function: num_queues
@@ -1246,11 +1291,7 @@ inline size_t Executor::num_queues() const noexcept {
 
 // Function: num_topologies
 inline size_t Executor::num_topologies() const {
-#if __cplusplus >= TF_CPP20
   return _num_topologies.load(std::memory_order_relaxed);
-#else
-  return _num_topologies;
-#endif
 }
 
 // Function: num_taskflows
@@ -1275,19 +1316,15 @@ inline void Executor::_spawn(size_t N) {
 
   for(size_t id=0; id<N; ++id) {
     _workers[id]._id = id;
-    _workers[id]._vtm = id;
-    _workers[id]._executor = this;
-    _workers[id]._waiter = &_notifier._waiters[id];
+    _workers[id]._sticky_victim = id;
     _workers[id]._thread = std::thread([&, &w=_workers[id]] () {
 
       // initialize the random engine and seed for work-stealing loop
-      w._rdgen.seed(static_cast<std::default_random_engine::result_type>(
-        std::hash<std::thread::id>()(std::this_thread::get_id()))
-      );
+      w._rdgen.seed(static_cast<uint32_t>(std::hash<std::thread::id>()(std::this_thread::get_id())));
 
       // before entering the work-stealing loop, call the scheduler prologue
-      if(_worker_interface) {
-        _worker_interface->scheduler_prologue(w);
+      if(_worker_if) {
+        _worker_if->scheduler_prologue(w);
       }
 
 
@@ -1321,71 +1358,45 @@ inline void Executor::_spawn(size_t N) {
 #endif
       
       // call the user-specified epilogue function
-      if(_worker_interface) {
-        _worker_interface->scheduler_epilogue(w, ptr);
+      if(_worker_if) {
+        _worker_if->scheduler_epilogue(w, ptr);
       }
 
     });
-
+    
+    // We avoid using thread-local storage to track the mapping between a thread
+    // and its corresponding worker in an executor. On Windows, thread-local
+    // storage can be unreliable in certain situations (see issue #727).
+    //
+    // Instead, we maintain a per-executor mapping from threads to workers.
+    // This approach has an additional advantage: according to the C++ Standard,
+    // std::thread::id uniquely identifies a thread object. Therefore, once the map
+    // returns a valid worker, we can be certain that the worker belongs to this
+    // executor. This eliminates the need for additional executor validation 
+    // required by using thread-local storage.
+    //
+    // Example:
+    //
+    //   Worker* w = this_worker();
+    //   // Using thread-local storage, we would need additional executor validation:
+    //   if (w == nullptr || w->_executor != this) { /* caller is not a worker of this executor */ }
+    //
+    //   // Using per-executor mapping, it suffices to check:
+    //   if (w == nullptr) { /* caller is not a worker of this executor */ }
+    //
     _t2w.emplace(_workers[id]._thread.get_id(), &_workers[id]);
   }
 }
 
-// Function: _corun_until
-template <typename P>
-void Executor::_corun_until(Worker& w, P&& stop_predicate) {
-
-  const size_t MAX_STEALS = ((num_queues() + 1) << 1);
-    
-  std::uniform_int_distribution<size_t> udist(0, num_queues()-1);
-  
-  exploit:
-
-  while(!stop_predicate()) {
-    
-    // here we don't do while-loop to drain out the local queue as it can
-    // potentially enter a very deep recursive corun, cuasing stack overflow
-    if(auto t = w._wsq.pop(); t) {
-      _invoke(w, t);
-    }
-    else {
-      size_t num_steals = 0;
-      size_t vtm = w._vtm;
-
-      explore:
-
-      t = (vtm < _workers.size()) ? _workers[vtm]._wsq.steal() : 
-                                    _buffers.steal(vtm - _workers.size());
-
-      if(t) {
-        _invoke(w, t);
-        w._vtm = vtm;
-        goto exploit;
-      }
-      else if(!stop_predicate()) {
-        if(++num_steals > MAX_STEALS) {
-          std::this_thread::yield();
-        }
-        vtm = udist(w._rdgen);
-        goto explore;
-      }
-      else {
-        break;
-      }
-    }
-  }
-}
-
 // Function: _explore_task
 inline bool Executor::_explore_task(Worker& w, Node*& t) {
 
   //assert(!t);
-  
-  const size_t MAX_STEALS = ((num_queues() + 1) << 1);
-  std::uniform_int_distribution<size_t> udist(0, num_queues()-1);
+  const size_t MAX_VICTIM = num_queues();
+  const size_t MAX_STEALS = ((MAX_VICTIM + 1) << 1);
 
   size_t num_steals = 0;
-  size_t vtm = w._vtm;
+  size_t vtm = w._sticky_victim;
 
   // Make the worker steal immediately from the assigned victim.
   while(true) {
@@ -1394,10 +1405,10 @@ inline bool Executor::_explore_task(Worker& w, Node*& t) {
     // Otherwise, steal from the buffer, adjusting the victim index based on the worker pool size.
     t = (vtm < _workers.size())
       ? _workers[vtm]._wsq.steal()
-      : _buffers.steal(vtm - _workers.size());
+      : _buffers[vtm - _workers.size()].queue.steal();
 
     if(t) {
-      w._vtm = vtm;
+      w._sticky_victim = vtm;
       break;
     }
 
@@ -1410,16 +1421,12 @@ inline bool Executor::_explore_task(Worker& w, Node*& t) {
       }
     }
 
-  #if __cplusplus >= TF_CPP20
     if(w._done.test(std::memory_order_relaxed)) {
-  #else
-    if(w._done.load(std::memory_order_relaxed)) {
-  #endif
       return false;
     } 
 
     // Randomely generate a next victim.
-    vtm = udist(w._rdgen); //w._rdvtm();
+    vtm = w._rdgen() % MAX_VICTIM;
   } 
   return true;
 }
@@ -1446,51 +1453,39 @@ inline bool Executor::_wait_for_task(Worker& w, Node*& t) {
     return true;
   }
 
-  // Entering the 2PC guard as all queues should be empty after many stealing attempts.
-  _notifier.prepare_wait(w._waiter);
+  // Entering the 2PC guard as all queues are likely empty after many stealing attempts.
+  _notifier.prepare_wait(w._id);
   
   // Condition #1: buffers should be empty
-  for(size_t vtm=0; vtm<_buffers.size(); ++vtm) {
-    if(!_buffers._buckets[vtm].queue.empty()) {
-      _notifier.cancel_wait(w._waiter);
-      w._vtm = vtm + _workers.size();
+  for(size_t b=0; b<_buffers.size(); ++b) {
+    if(!_buffers[b].queue.empty()) {
+      _notifier.cancel_wait(w._id);
+      w._sticky_victim = b + _workers.size();
       goto explore_task;
     }
   }
   
   // Condition #2: worker queues should be empty
   // Note: We need to use index-based looping to avoid data race with _spawan
-  // which initializes other worker data structure at the same time
-  for(size_t vtm=0; vtm<w._id; ++vtm) {
-    if(!_workers[vtm]._wsq.empty()) {
-      _notifier.cancel_wait(w._waiter);
-      w._vtm = vtm;
-      goto explore_task;
-    }
-  }
-  
-  // due to the property of the work-stealing queue, we don't need to check
-  // the queue of this worker
-  for(size_t vtm=w._id+1; vtm<_workers.size(); vtm++) {
-    if(!_workers[vtm]._wsq.empty()) {
-      _notifier.cancel_wait(w._waiter);
-      w._vtm = vtm;
+  // which initializes other worker data structure at the same time.
+  // Also, due to the property of a work-stealing queue, we don't need to check 
+  // this worker's work-stealing queue.
+  for(size_t k=0; k<_workers.size()-1; ++k) {
+    if(size_t vtm = k + (k >= w._id); !_workers[vtm]._wsq.empty()) {
+      _notifier.cancel_wait(w._id);
+      w._sticky_victim = vtm;
       goto explore_task;
     }
   }
   
   // Condition #3: worker should be alive
-#if __cplusplus >= TF_CPP20
   if(w._done.test(std::memory_order_relaxed)) {
-#else
-  if(w._done.load(std::memory_order_relaxed)) {
-#endif
-    _notifier.cancel_wait(w._waiter);
+    _notifier.cancel_wait(w._id);
     return false;
   }
   
   // Now I really need to relinquish myself to others.
-  _notifier.commit_wait(w._waiter);
+  _notifier.commit_wait(w._id);
   goto explore_task;
 }
 
@@ -1530,88 +1525,96 @@ inline size_t Executor::num_observers() const noexcept {
   return _observers.size();
 }
 
+// Procedure: _spill
+inline void Executor::_spill(Node* item) {
+  // Since pointers are aligned to 8 bytes, we perform a simple hash to avoid 
+  // contention caused by hashing to the same slot.
+  auto b = (reinterpret_cast<uintptr_t>(item) >> 16) % _buffers.size();
+  std::scoped_lock lock(_buffers[b].mutex);
+  _buffers[b].queue.push(item);
+}
+
+// Procedure: _bulk_spill
+template <typename I>
+void Executor::_bulk_spill(I first, size_t N) {
+  // assert(N != 0);
+  // Since pointers are aligned to 8 bytes, we perform a simple hash to avoid 
+  // contention caused by hashing to the same slot.
+  auto p = reinterpret_cast<uintptr_t>(*first) >> 16;
+  auto b = (p ^ (N << 6)) % _buffers.size();
+  std::scoped_lock lock(_buffers[b].mutex);
+  _buffers[b].queue.bulk_push(first, N);
+}
+
 // Procedure: _schedule
 inline void Executor::_schedule(Worker& worker, Node* node) {
-  
-  // caller is a worker of this executor - starting at v3.5 we do not use
-  // any complicated notification mechanism as the experimental result
-  // has shown no significant advantage.
-  if(worker._executor == this) {
-    worker._wsq.push(node, [&](){ _buffers.push(node); });
-    _notifier.notify_one();
-    return;
+  // starting at v3.5 we do not use any complicated notification mechanism 
+  // as the experimental result has shown no significant advantage.
+  if(worker._wsq.try_push(node) == false) {
+    _spill(node);
   }
-  
-  // caller is not a worker of this executor - go through the centralized queue
-  _buffers.push(node);
   _notifier.notify_one();
 }
 
 // Procedure: _schedule
 inline void Executor::_schedule(Node* node) {
-  _buffers.push(node);
+  _spill(node);
   _notifier.notify_one();
 }
 
 // Procedure: _schedule
 template <typename I>
-void Executor::_schedule(Worker& worker, I first, I last) {
+void Executor::_bulk_schedule(Worker& worker, I first, size_t num_nodes) {
 
-  size_t num_nodes = last - first;
-  
   if(num_nodes == 0) {
     return;
   }
-  
+
   // NOTE: We cannot use first/last in the for-loop (e.g., for(; first != last; ++first)).
   // This is because when a node v is inserted into the queue, v can run and finish 
   // immediately. If v is the last node in the graph, it will tear down the parent task vector
   // which cause the last ++first to fail. This problem is specific to MSVC which has a stricter
   // iterator implementation in std::vector than GCC/Clang.
-  if(worker._executor == this) {
-    for(size_t i=0; i<num_nodes; i++) {
-      auto node = detail::get_node_ptr(first[i]);
-      worker._wsq.push(node, [&](){ _buffers.push(node); });
-      _notifier.notify_one();
-    }
-    return;
-  }
-  
-  // caller is not a worker of this executor - go through the centralized queue
-  for(size_t i=0; i<num_nodes; i++) {
-    _buffers.push(detail::get_node_ptr(first[i]));
+  if(auto n = worker._wsq.try_bulk_push(first, num_nodes); n != num_nodes) {
+    _bulk_spill(first + n, num_nodes - n);
   }
   _notifier.notify_n(num_nodes);
+    
+  // notify first before spilling to hopefully wake up workers earlier 
+  // however, the experiment does not show any benefit for doing this.
+  //auto n = worker._wsq.try_bulk_push(first, num_nodes);
+  //_notifier.notify_n(n);
+  //_bulk_schedule(first + n, num_nodes - n);
 }
 
 // Procedure: _schedule
 template <typename I>
-inline void Executor::_schedule(I first, I last) {
+inline void Executor::_bulk_schedule(I first, size_t num_nodes) {
   
-  size_t num_nodes = last - first;
-
   if(num_nodes == 0) {
     return;
   }
-
+  
   // NOTE: We cannot use first/last in the for-loop (e.g., for(; first != last; ++first)).
   // This is because when a node v is inserted into the queue, v can run and finish 
   // immediately. If v is the last node in the graph, it will tear down the parent task vector
   // which cause the last ++first to fail. This problem is specific to MSVC which has a stricter
   // iterator implementation in std::vector than GCC/Clang.
-  for(size_t i=0; i<num_nodes; i++) {
-    _buffers.push(detail::get_node_ptr(first[i]));
-  }
+  _bulk_spill(first, num_nodes);
   _notifier.notify_n(num_nodes);
 }
   
+// Function: _schedule_graph_with_parent
 template <typename I>
-void Executor::_schedule_graph_with_parent(Worker& worker, I beg, I end, Node* parent) {
-  auto send = _set_up_graph(beg, end, parent->_topology, parent);
-  parent->_join_counter.fetch_add(send - beg, std::memory_order_relaxed);
-  _schedule(worker, beg, send);
+void Executor::_schedule_graph_with_parent(
+  Worker& worker, I beg, I end, Topology* tpg, NodeBase* parent
+) {
+  size_t num_srcs = (_set_up_graph(beg, end, tpg, parent) - beg);
+  parent->_join_counter.fetch_add(num_srcs, std::memory_order_relaxed);
+  _bulk_schedule(worker, beg, num_srcs);
 }
 
+// Function: _update_cache
 TF_FORCE_INLINE void Executor::_update_cache(Worker& worker, Node*& cache, Node* node) {
   if(cache) {
     _schedule(worker, cache);
@@ -1637,10 +1640,10 @@ inline void Executor::_invoke(Worker& worker, Node* node) {
     goto invoke_task;
   }
 
-  // if the work has been cancelled, there is no need to continue
+  // If the work has been cancelled, there is no need to continue.
   // Here, we do tear_down_invoke since async tasks may also get cancelled where
   // we need to recycle the node.
-  if(node->_is_cancelled()) {
+  if(node->_is_parent_cancelled()) {
     _tear_down_invoke(worker, node, cache);
     TF_INVOKE_CONTINUATION();
     return;
@@ -1650,7 +1653,7 @@ inline void Executor::_invoke(Worker& worker, Node* node) {
   if(node->_semaphores && !node->_semaphores->to_acquire.empty()) {
     SmallVector<Node*> waiters;
     if(!node->_acquire_all(waiters)) {
-      _schedule(worker, waiters.begin(), waiters.end());
+      _bulk_schedule(worker, waiters.begin(), waiters.size());
       return;
     }
   }
@@ -1740,7 +1743,7 @@ inline void Executor::_invoke(Worker& worker, Node* node) {
   if(node->_semaphores && !node->_semaphores->to_release.empty()) {
     SmallVector<Node*> waiters;
     node->_release_all(waiters);
-    _schedule(worker, waiters.begin(), waiters.end());
+    _bulk_schedule(worker, waiters.begin(), waiters.size());
   }
 
   // Reset the join counter with strong dependencies to support cycles.
@@ -1803,8 +1806,9 @@ inline void Executor::_tear_down_nonasync(Worker& worker, Node* node, Node*& cac
     // the node may be deleted
     auto state = parent->_nstate;
     if(parent->_join_counter.fetch_sub(1, std::memory_order_acq_rel) == 1) {
+      // this task is spawned from a preempted parent, so we need to resume it
       if(state & NSTATE::PREEMPTED) {
-        _update_cache(worker, cache, parent);
+        _update_cache(worker, cache, static_cast<Node*>(parent));
       }
     }
   }
@@ -1844,38 +1848,51 @@ inline void Executor::_observer_epilogue(Worker& worker, Node* node) {
 // Procedure: _process_exception
 inline void Executor::_process_exception(Worker&, Node* node) {
 
-  constexpr static auto flag = ESTATE::EXCEPTION | ESTATE::CANCELLED;
-
-  // find the anchor and mark the entire path with exception so recursive
-  // or nested tasks can be cancelled properly
-  // since exception can come from asynchronous task (with runtime), the node
-  // itself can be anchored
-  auto anchor = node;
-  while(anchor && (anchor->_estate.load(std::memory_order_relaxed) & ESTATE::ANCHORED) == 0) {
-    anchor->_estate.fetch_or(flag, std::memory_order_relaxed);
-    anchor = anchor->_parent;
+  // Finds the anchor and mark the entire path with exception, 
+  // so recursive tasks can be cancelled properly.
+  // Since exception can come from asynchronous task (with runtime), the node itself can be anchored.
+  NodeBase* ea = node;     // explicit anchor
+  NodeBase* ia = nullptr;  // implicit anchor
+  
+  while(ea && (ea->_estate.load(std::memory_order_relaxed) & ESTATE::EXPLICITLY_ANCHORED) == 0) {
+    ea->_estate.fetch_or(ESTATE::EXCEPTION, std::memory_order_relaxed);
+    // e only want the inner-most implicit anchor
+    if(ia == nullptr && (ea->_nstate & NSTATE::IMPLICITLY_ANCHORED)) {
+      ia = ea;
+    }
+    ea = ea->_parent;
   }
+  
+  // flag used to ensure execution is caught in a thread-safe manner
+  constexpr static auto flag = ESTATE::EXCEPTION | ESTATE::CAUGHT;
 
-  // the exception occurs under a blocking call (e.g., corun, join)
-  if(anchor) {
+  // The exception occurs under a blocking call (e.g., corun, join).
+  if(ea) {
     // multiple tasks may throw, and we only take the first thrown exception
-    if((anchor->_estate.fetch_or(flag, std::memory_order_relaxed) & ESTATE::EXCEPTION) == 0) {
-      anchor->_exception_ptr = std::current_exception();
+    if((ea->_estate.fetch_or(flag, std::memory_order_relaxed) & ESTATE::CAUGHT) == 0) {
+      ea->_exception_ptr = std::current_exception();
       return;
     }
   }
   // otherwise, we simply store the exception in the topology and cancel it
   else if(auto tpg = node->_topology; tpg) {
     // multiple tasks may throw, and we only take the first thrown exception
-    if((tpg->_estate.fetch_or(flag, std::memory_order_relaxed) & ESTATE::EXCEPTION) == 0) {
+    if((tpg->_estate.fetch_or(flag, std::memory_order_relaxed) & ESTATE::CAUGHT) == 0) {
       tpg->_exception_ptr = std::current_exception();
       return;
     }
   }
+  // Implicit anchor has the lowest priority
+  else if(ia){
+    if((ia->_estate.fetch_or(flag, std::memory_order_relaxed) & ESTATE::CAUGHT) == 0) {
+      ia->_exception_ptr = std::current_exception();
+      return;
+    }
+  }
   
-  // for now, we simply store the exception in this node; this can happen in an 
+  // For now, we simply store the exception in this node; this can happen in an 
   // execution that does not have any external control to capture the exception,
-  // such as silent async task
+  // such as silent async task without any parent.
   node->_exception_ptr = std::current_exception();
 }
 
@@ -1914,7 +1931,7 @@ inline bool Executor::_invoke_subflow_task(Worker& worker, Node* node) {
       node->_nstate |= NSTATE::PREEMPTED;
 
       // set up and schedule the graph
-      _schedule_graph_with_parent(worker, g.begin(), g.end(), node);
+      _schedule_graph_with_parent(worker, g.begin(), g.end(), node->_topology, node);
       return true;
     }
   }
@@ -1922,7 +1939,10 @@ inline bool Executor::_invoke_subflow_task(Worker& worker, Node* node) {
     node->_nstate &= ~NSTATE::PREEMPTED;
   }
 
-  // the subflow has finished or joined
+  // The subflow has finished or joined.
+  // By default, we clear the subflow storage as applications can perform recursive
+  // subflow tasking which accumulates a huge amount of memory overhead, hampering 
+  // the performance.
   if((node->_nstate & NSTATE::RETAIN_SUBFLOW) == 0) {
     g.clear();
   }
@@ -1970,7 +1990,7 @@ inline bool Executor::_invoke_module_task_impl(Worker& w, Node* node, Graph& gra
   if((node->_nstate & NSTATE::PREEMPTED) == 0) {
     // signal the executor to preempt this node
     node->_nstate |= NSTATE::PREEMPTED;
-    _schedule_graph_with_parent(w, graph.begin(), graph.end(), node);
+    _schedule_graph_with_parent(w, graph.begin(), graph.end(), node->_topology, node);
     return true;
   }
 
@@ -2108,15 +2128,10 @@ tf::Future<void> Executor::run_until(Taskflow& f, P&& p, C&& c) {
 
   _increment_topology();
 
-  //// Need to check the empty under the lock since subflow task may
-  //// define detached blocks that modify the taskflow at the same time
-  //bool empty;
-  //{
-  //  std::lock_guard<std::mutex> lock(f._mutex);
-  //  empty = f.empty();
-  //}
-
-  // No need to create a real topology but returns an dummy future
+  // No need to create a real topology but returns an dummy future for invariant.
+  // Note that here we don't do std::async(std::launch::deferred, [](){}) because
+  // the invariant requires the future to be ready before decrementing the topology,
+  // rather than calling future.get() from the caller.
   if(f.empty() || p()) {
     c();
     std::promise<void> promise;
@@ -2126,7 +2141,8 @@ tf::Future<void> Executor::run_until(Taskflow& f, P&& p, C&& c) {
   }
 
   // create a topology for this run
-  auto t = std::make_shared<Topology>(f, std::forward<P>(p), std::forward<C>(c));
+  //auto t = std::make_shared<Topology>(f, std::forward<P>(p), std::forward<C>(c));
+  auto t = std::make_shared<DerivedTopology<P, C>>(f, std::forward<P>(p), std::forward<C>(c));
 
   // need to create future before the topology got torn down quickly
   tf::Future<void> future(t->_promise.get_future(), t);
@@ -2158,6 +2174,63 @@ tf::Future<void> Executor::run_until(Taskflow&& f, P&& pred, C&& c) {
   return run_until(*itr, std::forward<P>(pred), std::forward<C>(c));
 }
 
+// Function: corun_until
+template <typename P>
+void Executor::corun_until(P&& predicate) {
+  
+  Worker* w = this_worker();
+  if(w == nullptr) {
+    TF_THROW("corun_until must be called by a worker of the executor");
+  }
+
+  _corun_until(*w, std::forward<P>(predicate));
+}
+
+// Function: _corun_until
+template <typename P>
+void Executor::_corun_until(Worker& w, P&& stop_predicate) {
+
+  const size_t MAX_VICTIM = num_queues();
+  const size_t MAX_STEALS = ((MAX_VICTIM + 1) << 1);
+    
+  exploit:
+
+  while(!stop_predicate()) {
+    
+    // here we don't do while-loop to drain out the local queue as it can
+    // potentially enter a very deep recursive corun, cuasing stack overflow
+    if(auto t = w._wsq.pop(); t) {
+      _invoke(w, t);
+    }
+    else {
+      size_t num_steals = 0;
+      size_t vtm = w._sticky_victim;
+
+      explore:
+
+      t = (vtm < _workers.size()) 
+        ? _workers[vtm]._wsq.steal()
+        : _buffers[vtm-_workers.size()].queue.steal();
+
+      if(t) {
+        _invoke(w, t);
+        w._sticky_victim = vtm;
+        goto exploit;
+      }
+      else if(!stop_predicate()) {
+        if(++num_steals > MAX_STEALS) {
+          std::this_thread::yield();
+        }
+        vtm = w._rdgen() % MAX_VICTIM;
+        goto explore;
+      }
+      else {
+        break;
+      }
+    }
+  }
+}
+
 // Function: corun
 template <typename T>
 void Executor::corun(T& target) {
@@ -2165,29 +2238,17 @@ void Executor::corun(T& target) {
   static_assert(has_graph_v<T>, "target must define a member function 'Graph& graph()'");
   
   Worker* w = this_worker();
-  if(w == nullptr || w->_executor != this) {
+  if(w == nullptr) {
     TF_THROW("corun must be called by a worker of the executor");
   }
 
-  Node anchor;
-  _corun_graph(*w, &anchor, target.graph().begin(), target.graph().end());
-}
-
-// Function: corun_until
-template <typename P>
-void Executor::corun_until(P&& predicate) {
-  
-  Worker* w = this_worker();
-  if(w == nullptr || w->_executor != this) {
-    TF_THROW("corun_until must be called by a worker of the executor");
-  }
-
-  _corun_until(*w, std::forward<P>(predicate));
+  NodeBase anchor;
+  _corun_graph(*w, nullptr, &anchor, target.graph().begin(), target.graph().end());
 }
 
 // Procedure: _corun_graph
 template <typename I>
-void Executor::_corun_graph(Worker& w, Node* p, I first, I last) {
+void Executor::_corun_graph(Worker& w, Topology* tpg, NodeBase* p, I first, I last) {
 
   // empty graph
   if(first == last) {
@@ -2196,53 +2257,36 @@ void Executor::_corun_graph(Worker& w, Node* p, I first, I last) {
   
   // anchor this parent as the blocking point
   {
-    AnchorGuard anchor(p);
-    _schedule_graph_with_parent(w, first, last, p);
+    ExplicitAnchorGuard anchor(p);
+    _schedule_graph_with_parent(w, first, last, tpg, p);
     _corun_until(w, [p] () -> bool { 
       return p->_join_counter.load(std::memory_order_acquire) == 0; }
     );
   }
 
-  // rethrow the exception to the blocker
+  // rethrow the exception to the caller
   p->_rethrow_exception();
 }
 
 // Procedure: _increment_topology
 inline void Executor::_increment_topology() {
-#if __cplusplus >= TF_CPP20
   _num_topologies.fetch_add(1, std::memory_order_relaxed);
-#else
-  std::lock_guard<std::mutex> lock(_topology_mutex);
-  ++_num_topologies;
-#endif
 }
 
 // Procedure: _decrement_topology
 inline void Executor::_decrement_topology() {
-#if __cplusplus >= TF_CPP20
   if(_num_topologies.fetch_sub(1, std::memory_order_acq_rel) == 1) {
     _num_topologies.notify_all();
   }
-#else
-  std::lock_guard<std::mutex> lock(_topology_mutex);
-  if(--_num_topologies == 0) {
-    _topology_cv.notify_all();
-  }
-#endif
 }
 
 // Procedure: wait_for_all
 inline void Executor::wait_for_all() {
-#if __cplusplus >= TF_CPP20
   size_t n = _num_topologies.load(std::memory_order_acquire);
   while(n != 0) {
     _num_topologies.wait(n, std::memory_order_acquire);
     n = _num_topologies.load(std::memory_order_acquire);
   }
-#else
-  std::unique_lock<std::mutex> lock(_topology_mutex);
-  _topology_cv.wait(lock, [&](){ return _num_topologies == 0; });
-#endif
 }
 
 // Function: _set_up_topology
@@ -2251,20 +2295,20 @@ inline void Executor::_set_up_topology(Worker* w, Topology* tpg) {
   // ---- under taskflow lock ----
   auto& g = tpg->_taskflow._graph;
   
-  auto send = _set_up_graph(g.begin(), g.end(), tpg, nullptr);
-  tpg->_join_counter.store(send - g.begin(), std::memory_order_relaxed);
+  size_t num_srcs = (_set_up_graph(g.begin(), g.end(), tpg, nullptr) - g.begin());
+  tpg->_join_counter.store(num_srcs, std::memory_order_relaxed);
 
-  w ? _schedule(*w, g.begin(), send) : _schedule(g.begin(), send);
+  w ? _bulk_schedule(*w, g.begin(), num_srcs) : _bulk_schedule(g.begin(), num_srcs);
 }
 
 // Function: _set_up_graph
 template <typename I>
-I Executor::_set_up_graph(I first, I last, Topology* tpg, Node* parent) {
+I Executor::_set_up_graph(I first, I last, Topology* tpg, NodeBase* parent) {
 
   auto send = first;
   for(; first != last; ++first) {
 
-    auto node = first->get();
+    auto node = *first;
     node->_topology = tpg;
     node->_parent = parent;
     node->_nstate = NSTATE::NONE;
@@ -2289,7 +2333,7 @@ inline void Executor::_tear_down_topology(Worker& worker, Topology* tpg) {
   //assert(&tpg == &(f._topologies.front()));
 
   // case 1: we still need to run the topology again
-  if(!tpg->_exception_ptr && !tpg->cancelled() && !tpg->_pred()) {
+  if(!tpg->_exception_ptr && !tpg->cancelled() && !tpg->predicate()) {
     //assert(tpg->_join_counter == 0);
     std::lock_guard<std::mutex> lock(f._mutex);
     _set_up_topology(&worker, tpg);
@@ -2298,9 +2342,7 @@ inline void Executor::_tear_down_topology(Worker& worker, Topology* tpg) {
   else {
 
     // invoke the callback after each run
-    if(tpg->_call != nullptr) {
-      tpg->_call();
-    }
+    tpg->on_finish();
 
     // If there is another run (interleave between lock)
     if(std::unique_lock<std::mutex> lock(f._mutex); f._topologies.size()>1) {
@@ -2354,10 +2396,10 @@ inline void Subflow::join() {
     TF_THROW("subflow already joined");
   }
     
-  _executor._corun_graph(_worker, _parent, _graph.begin(), _graph.end());
+  _executor._corun_graph(_worker, _node->_topology, _node, _graph.begin(), _graph.end());
   
   // join here since corun graph may throw exception
-  _parent->_nstate |= NSTATE::JOINED_SUBFLOW;
+  _node->_nstate |= NSTATE::JOINED_SUBFLOW;
 }
 
 #endif
