@@ -29,25 +29,8 @@
   #define TF_NO_INLINE
 #endif
 
-// ============================================================================
-// likely and unlikely
-// ============================================================================
-
-#if defined(__GNUC__)
-  #define TF_LIKELY(x) (__builtin_expect((x), 1))
-  #define TF_UNLIKELY(x) (__builtin_expect((x), 0))
-#else
-  #define TF_LIKELY(x) (x)
-  #define TF_UNLIKELY(x) (x)
-#endif
-
-
-
 // ----------------------------------------------------------------------------    
 
-#define TF_FWD(T, x) std::forward<T>(x)
-
-
 
 
 
